package net.wanji.web.service.impl;

import net.wanji.feign.pojo.result.JsonViewObject;
import net.wanji.feign.pojo.vo.ScheduleSendVO;
import net.wanji.feign.pojo.vo.SchemeSendVO;
import net.wanji.feign.service.UtcFeignClients;
import net.wanji.web.dto.ScheduleIdDTO;
import net.wanji.web.dto.SchemeIdDTO;
import net.wanji.web.mapper.scheme.CrossPhaseMapper;
import net.wanji.web.mapper.scheme.CrossPlanMapper;
import net.wanji.web.mapper.scheme.CrossSchedulesPlanMapper;
import net.wanji.web.mapper.scheme.CrossSchemeMapper;
import net.wanji.web.po.scheme.CrossPhasePO;
import net.wanji.web.po.scheme.CrossPlanPO;
import net.wanji.web.po.scheme.CrossSchedulesPlanPO;
import net.wanji.web.po.scheme.CrossSchemePO;
import net.wanji.web.service.SchemeSendService;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/2/1 9:39
 */
@Service
public class SchemeSendServiceImpl implements SchemeSendService {
    private final UtcFeignClients utcFeignClients;
    private final CrossSchedulesPlanMapper crossSchedulesPlanMapper;
    private final CrossPlanMapper crossPlanMapper;
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossPhaseMapper crossPhaseMapper;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

    public SchemeSendServiceImpl(@Qualifier("net.wanji.feign.service.UtcFeignClients") UtcFeignClients utcFeignClients, CrossSchedulesPlanMapper crossSchedulesPlanMapper, CrossPlanMapper crossPlanMapper, CrossSchemeMapper crossSchemeMapper, CrossPhaseMapper crossPhaseMapper) {
        this.utcFeignClients = utcFeignClients;
        this.crossSchedulesPlanMapper = crossSchedulesPlanMapper;
        this.crossPlanMapper = crossPlanMapper;
        this.crossSchemeMapper = crossSchemeMapper;
        this.crossPhaseMapper = crossPhaseMapper;
    }

    @Override
    public JsonViewObject scheduleSend(ScheduleIdDTO scheduleIdDTO) {
        String crossId = scheduleIdDTO.getCrossId();
        ScheduleSendVO scheduleSendVO = new ScheduleSendVO();
        scheduleSendVO.setCrossCode(crossId);
        // 构造时间表
        List<ScheduleSendVO.Schedule> schedules = buildSchedules(crossId, scheduleIdDTO);
        scheduleSendVO.setSchedules(schedules);
        // 发送请求
        JsonViewObject jsonViewObject = utcFeignClients.scheduleSend(scheduleSendVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-时间表下发UTC服务调用异常");
        }
        return jsonViewObject.success();
    }

    @Override
    public JsonViewObject schemeSend(SchemeIdDTO schemeIdDTO) {
        String crossId = schemeIdDTO.getCrossId();
        SchemeSendVO schemeSendVO = new SchemeSendVO();
        schemeSendVO.setCrossCode(crossId);
        // 构造方案列表
        Integer schemeId = schemeIdDTO.getSchemeId();
        List<SchemeSendVO.Pattern> patternList = buildPatternList(crossId, schemeId);
        schemeSendVO.setPatternList(patternList);
        // 发送请求
        JsonViewObject jsonViewObject = utcFeignClients.schemeSend(schemeSendVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-基础方案下发UTC服务调用异常");
        }
        return jsonViewObject.success();
    }

    private List<SchemeSendVO.Pattern> buildPatternList(String crossId, Integer schemeId) {
        List<SchemeSendVO.Pattern> patternList = new ArrayList<>();
        CrossSchemePO crossSchemePO = crossSchemeMapper.selectById(schemeId);
        SchemeSendVO.Pattern pattern = new SchemeSendVO.Pattern();
        pattern.setPatternNo(crossSchemePO.getSchemeNo());
        pattern.setPatternName(crossSchemePO.getName());
        pattern.setCycle(crossSchemePO.getCycle().toString());
        pattern.setCoordPhase(crossSchemePO.getCoordPhase().toString());
        pattern.setOffset(crossSchemePO.getOffset().toString());
        // 构造环列表
        List<SchemeSendVO.Pattern.Ring> rings = buildRings(crossId, schemeId);
        pattern.setRings(rings);
        patternList.add(pattern);

        return patternList;
    }

    private List<SchemeSendVO.Pattern.Ring> buildRings(String crossId, Integer schemeId) {
        List<SchemeSendVO.Pattern.Ring> rings = new ArrayList<>();
        // 根据路口ID和方案ID查询相位集合，根据ringNo分组，特殊控制ringNo为0
        List<CrossPhasePO> crossPhasePOList = crossPhaseMapper.selectByCrossIdAndSchemeId(crossId, schemeId);
        Map<Integer, List<CrossPhasePO>> collectMap = crossPhasePOList.stream()
                .collect(Collectors.groupingBy(CrossPhasePO::getRingNo));
        for (Map.Entry<Integer, List<CrossPhasePO>> entry : collectMap.entrySet()) {
            SchemeSendVO.Pattern.Ring ring = new SchemeSendVO.Pattern.Ring();
            ring.setRingNo(entry.getKey().toString());
            // 构造相位列表
            List<CrossPhasePO> phases = entry.getValue();
            List<SchemeSendVO.Pattern.Ring.Phase> phaseList = buildPhaseList(phases);
            ring.setPhaseList(phaseList);
            rings.add(ring);
        }
        return rings;
    }

    private List<SchemeSendVO.Pattern.Ring.Phase> buildPhaseList(List<CrossPhasePO> phases) {
        List<SchemeSendVO.Pattern.Ring.Phase> phaseList = new ArrayList<>();
        for (CrossPhasePO phasePO : phases) {
            SchemeSendVO.Pattern.Ring.Phase phase = new SchemeSendVO.Pattern.Ring.Phase();
            phase.setPhaseNo(phasePO.getPhaseNo());
            phase.setPhaseName(phasePO.getName());
            phase.setSort(phasePO.getSort().toString());
            phase.setControlMode(phasePO.getControlMode().toString());
            phase.setMinGreenTime(phasePO.getMinGreenTime().toString());
            phase.setMaxGreenTime(phasePO.getMaxGreenTime().toString());
            phase.setPhaseTime(phasePO.getPhaseTime().toString());
            phase.setGreenTime(phasePO.getGreenTime().toString());
            phase.setGreenFlashTime(phasePO.getGreenFlashTime().toString());
            phase.setPedFlashTime(phasePO.getPedFlashTime().toString());
            phase.setYellowTime(phasePO.getYellowTime().toString());
            phase.setRedTime(phasePO.getRedTime().toString());

            phaseList.add(phase);
        }
        return phaseList;
    }

    private List<ScheduleSendVO.Schedule> buildSchedules(String crossId, ScheduleIdDTO scheduleIdDTO) {
        List<ScheduleSendVO.Schedule> schedules = new ArrayList<>();
        Integer scheduleId = scheduleIdDTO.getScheduleId();
        List<CrossSchedulesPlanPO> crossSchedulesPlanPOList =
                crossSchedulesPlanMapper.selectByCrossIdAndSchedulesId(crossId, scheduleId);
        ScheduleSendVO.Schedule schedule = new ScheduleSendVO.Schedule();
        List<ScheduleSendVO.Schedule.Week> weeks = new ArrayList<>();
        List<ScheduleSendVO.Schedule.SpecialDay> specialDays = new ArrayList<>();
        for (CrossSchedulesPlanPO crossSchedulesPlanPO : crossSchedulesPlanPOList) {
            Integer planId = crossSchedulesPlanPO.getPlanId();
            Integer week = crossSchedulesPlanPO.getWeek();
            Date specialDate = crossSchedulesPlanPO.getSpecialDate();
            if (week != 0) {
                // 添加至星期列表
                addToWeeks(weeks, week, planId);
            } else {
                // 添加至特殊日期列表
                addToSpecialDays(specialDays, specialDate, planId);
            }
        }
        schedule.setWeeks(weeks);
        schedule.setSpecialDays(specialDays);
        schedules.add(schedule);
        return schedules;
    }

    private void addToSpecialDays(List<ScheduleSendVO.Schedule.SpecialDay> specialDays,
                                  Date specialDate, Integer planId) {
        ScheduleSendVO.Schedule.SpecialDay specialDay = new ScheduleSendVO.Schedule.SpecialDay();
        specialDay.setDateStr(sdf.format(specialDate));
        String planNo = getPlanNo(planId);
        specialDay.setPlanNo(Integer.parseInt(planNo));
        specialDays.add(specialDay);
    }

    private void addToWeeks(List<ScheduleSendVO.Schedule.Week> weeks, Integer week, Integer planId) {
        ScheduleSendVO.Schedule.Week weekObj = new ScheduleSendVO.Schedule.Week();
        weekObj.setWeekNum(week);
        String planNo = getPlanNo(planId);
        weekObj.setPlanNo(Integer.parseInt(planNo));
        weeks.add(weekObj);
    }

    private String getPlanNo(Integer planId) {
        CrossPlanPO crossPlanPO = crossPlanMapper.selectById(planId);
        return crossPlanPO.getPlanNo();
    }
}
